﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.ComponentModel;
using System.Globalization;
using System.Windows;
using System.Windows.Controls;
using System.Collections.ObjectModel;

namespace AnnuityCalc {

	public class AnnuityData: INotifyPropertyChanged { 

		public decimal Summ {get; set;}
		public int PeriodsCount { get; set; }
		public string RateString { get; set; }

		/// <summary>
		/// Полная сумма выплат
		/// </summary>
		public decimal FullSumm {
			get {
				decimal full = 0;
				full = this._parts.Sum(e => e.AP);
				return Math.Round(full, 2);
			}
		}
		/// <summary>
		/// Полная сумма выплат по процентам
		/// </summary>
		public decimal FullPercent {
			get {
				return Math.Round(this.FullSumm > 0?(this.FullSumm - this.Summ):0, 2);
			}
		}
		/// <summary>
		/// Процент выплат процентов
		/// </summary>
		public decimal PercentPercent {
			get {
				if (this.FullSumm > 0)
					return Math.Round((this.FullSumm * 100 / this.Summ - 100), 2);
				else
					return 0;
			}
		}

		private IList<AnnuityPart> _parts = new List<AnnuityPart>();
		public IEnumerable<AnnuityPart> AnnuityParts {
			get {
				return _parts;
			}
		}

		public event PropertyChangedEventHandler PropertyChanged;

		const int DaysPerYear = 365;
		const int DaysPerMonth = 30;

		internal void ReCalc () {
			string temp_rate = this.RateString.Replace ('.', ',').Replace(",", CultureInfo.CurrentCulture.NumberFormat.CurrencyDecimalSeparator);
			double rate = Convert.ToDouble (temp_rate);	

			decimal dolg = this.Summ;
			double rate12 = rate / 12/100;
			double temp = Math.Pow(1+rate12, this.PeriodsCount);
			double K = rate12*temp/(temp - 1);
			decimal aAP = Math.Round(this.Summ * Convert.ToDecimal(K), 2);

			decimal ppp = 0;//percent per period
			decimal dpp = 0;//dolg per period

			this._parts = new List<AnnuityPart>(this.PeriodsCount);
			for (int i = 0; i < this.PeriodsCount; i++) {
				ppp = Math.Round(Convert.ToDecimal(((double)dolg) / 100 * rate / AnnuityData.DaysPerYear * AnnuityData.DaysPerMonth), 2);
				dpp = aAP - ppp;
				dolg -= dpp;

				_parts.Add(new AnnuityPart { Number = i + 1, AP = aAP, Percent = ppp, Dolg = dpp });
			}
			PropertyChanged(this, new PropertyChangedEventArgs("AnnuityParts"));
			PropertyChanged(this, new PropertyChangedEventArgs("FullSumm"));
			PropertyChanged(this, new PropertyChangedEventArgs("FullPercent"));
			PropertyChanged(this, new PropertyChangedEventArgs("PercentPercent"));
		}//ReCalc
	}

	public class AnnuityPart {
		public int Number {get;set;}
		public decimal AP {get;set;}
		public decimal Percent { get; set; }
		public decimal Dolg { get; set; }
	}

	public partial class Page : UserControl {

		private AnnuityData Data {
			get;
			set;
		}

		public Page() {

			InitializeComponent();

			this.Data = new AnnuityData { Summ = 600000, PeriodsCount = 60, RateString = "16" };
			this.DataContext = this.Data;

			RunButton.Focus();
		}

		void DoCalc(object sender, RoutedEventArgs e) {
			this.Data.ReCalc();
		}
	}//class Page
}//namespace
